/**
 *****************************************************************************************
 *	@file etn_mesh_fota.h
 *
 *	@brief eaton ble mesh fota model header file
 *
 *	@details this custom model from eaton, used for ble mesh ota
 *
 *	@version C Style Guide Version 1.0
 *
 *	@copyright 2022 Eaton Corporation. All Rights Reserved.
 *
 *****************************************************************************************
 */
#ifndef ETN_MESH_FOTA_H
#define ETN_MESH_FOTA_H

/*
 *****************************************************************************************
 *		Headers
 *****************************************************************************************
 */
#include <bluetooth/mesh/model_types.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 *****************************************************************************************
 *		Defines
 *****************************************************************************************
 */
#define BT_MESH_COMPANY_ID_EATON              0x035C

#define BT_MESH_MODEL_ID_ETN_FOTA_DIS_SRV     0x9001
#define BT_MESH_MODEL_ID_ETN_FOTA_DIS_CLI     0x9002
#define BT_MESH_MODEL_ID_ETN_FOTA_BLB_SRV     0x9003
#define BT_MESH_MODEL_ID_ETN_FOTA_BLB_CLI     0x9004
#define BT_MESH_MODEL_ID_ETN_FOTA_UPD_SRV     0x9005
#define BT_MESH_MODEL_ID_ETN_FOTA_UPD_CLI     0x9006

#define BT_MESH_FOTA_MAX_BYTE_TRANSFER  138

/*
 *****************************************************************************************
 *		Constants
 *****************************************************************************************
 */

/*
 *****************************************************************************************
 *		Variables
 *****************************************************************************************PLAT
 */

/*
 *****************************************************************************************
 *		Typedefs
 *****************************************************************************************
 */

#define BYTE_TO_BINARY_PATTERN "%c%c%c%c%c%c%c%c"
#define BYTE_TO_BINARY(byte)  \
  (byte & 0x80 ? '1' : '0'), \
  (byte & 0x40 ? '1' : '0'), \
  (byte & 0x20 ? '1' : '0'), \
  (byte & 0x10 ? '1' : '0'), \
  (byte & 0x08 ? '1' : '0'), \
  (byte & 0x04 ? '1' : '0'), \
  (byte & 0x02 ? '1' : '0'), \
  (byte & 0x01 ? '1' : '0') 
  
enum etn_fota_error_list{
    FOTA_ERROR_NONE,
    FOTA_ERROR_DATA_PARSE,
    FOTA_ERROR_DATA_VALIDATION
};

struct etn_fota_error{
    uint32_t opcode;
    uint8_t  err_code; 
}__packed;

/**
 * @brief function name
 * @param[in] param
 * @return return
 */

#ifdef __cplusplus
}
#endif

#endif