/**
 *****************************************************************************************
 *	@file etn_mesh_fota_dis_cli.h
 *
 *	@brief eaton ble mesh fota firmware distribution client model header file
 *
 *	@details this custom model from eaton, used for firmware distribution client model
 *
 *	@version C Style Guide Version 1.0
 *
 *	@copyright 2022 Eaton Corporation. All Rights Reserved.
 *
 *****************************************************************************************
 */
#ifndef ETN_MESH_FOTA_DIS_CLI_H
#define ETN_MESH_FOTA_DIS_CLI_H

/*
 *****************************************************************************************
 *		Headers
 *****************************************************************************************
 */
#include <bluetooth/mesh/model_types.h>
#include <etn_mesh_fota.h>
#include <etn_mesh_fota_dis.h>

#ifdef __cplusplus
extern "C" {
#endif

struct bt_mesh_etn_fota_dis_cli;

extern const struct bt_mesh_model_op _bt_mesh_etn_fota_dis_cli_op[];
extern const struct bt_mesh_model_cb _bt_mesh_etn_fota_dis_cli_cb;

/** @def BT_MESH_MODEL_ETN_FOTA_DIS_CLI
 * @brief BLE Mesh firmware distribution server model composition data entry.
 * @param[in] _dcli Pointer to a @ref bt_mesh_etn_fota_dis_cli instance from handler definition.
 */
#define BT_MESH_MODEL_ETN_FOTA_DIS_CLI( _discli )                                          \
	BT_MESH_MODEL_VND_CB( BT_MESH_COMPANY_ID_EATON,       \
						  BT_MESH_MODEL_ID_ETN_FOTA_DIS_CLI,                      \
						  _bt_mesh_etn_fota_dis_cli_op, &( _discli )->pub,                   \
						  BT_MESH_MODEL_USER_DATA( struct bt_mesh_etn_fota_dis_cli, _discli ),                        \
						  &_bt_mesh_etn_fota_dis_cli_cb )

/** BLE Mesh firmware distribution server model handlers' declaration. */
struct bt_mesh_etn_fota_dis_cli_handlers
{
	void( *const start )( struct bt_mesh_etn_fota_dis_cli *discli );

	void( *const dfu_start )( enum fota_dis_cmd_status *dfu_cmd_reply );

	void( *const metadata_get )( enum fota_dis_srv_metadata_status *metadata_reply );

	void( *const distribn_get )( enum fota_dis_list_status *dstrbn_list_reply );

	void( *const server_state_get )( enum fota_dis_srv_state *model_state );

	void ( *const error_reporting )( struct bt_mesh_etn_fota_dis_cli *discli, struct bt_mesh_msg_ctx *ctx, struct etn_fota_error *err_report );

};

/**
 * @brief	BLE Mesh firmware distribution server model context.
 * @details	Context structure of firmware distribution server model.
 */
struct bt_mesh_etn_fota_dis_cli
{
	struct bt_mesh_model *model;							// Access model pointer
	struct bt_mesh_model_pub pub;							// Publish parameters
	struct net_buf_simple pub_msg;							// Publication message
	uint8_t buf[BT_MESH_MODEL_BUF_LEN( BT_MESH_ETN_FOTA_DIS_IMAGE_METADATA_GET_OP, BT_MESH_ETN_FOTA_SRV_MESSAGE_MAXLEN )];	//
																														//
																														// Publication
																														//
																														// message
																														//
																														// buffer
	const struct bt_mesh_etn_fota_dis_cli_handlers *handlers;	// Handler function structure
};


int bt_mesh_etn_fota_dis_cli_metadata_send(struct bt_mesh_etn_fota_dis_cli *discli, uint16_t addr, struct fota_dis_metadata *metadata );
int bt_mesh_etn_fota_dis_cli_update_start_send(struct bt_mesh_etn_fota_dis_cli *discli, uint16_t addr);
int bt_mesh_etn_fota_dis_cli_receive_list_send(struct bt_mesh_etn_fota_dis_cli *discli, uint16_t addr, uint64_t *distribution_list);
int bt_mesh_etn_fota_dis_cli_state_get_send(struct bt_mesh_etn_fota_dis_cli *discli, uint16_t addr);

#ifdef __cplusplus
}
#endif

#endif